@@ -37,6 +37,7 @@ struct LxmlElementTagMatcher {
   PyObject_HEAD
   struct __pyx_vtabstruct_5etree__ElementTagMatcher *__pyx_vtab;
   PyObject *_pystrings;
+  int _node_type;
   char (*_href);
   char (*_name);
 };
@@ -70,6 +71,7 @@ static void ((*appendChild)(struct LxmlElement *,struct LxmlElement *));
 static PyObject *((*attributeValue)(xmlNode (*),xmlAttr (*)));
 static PyObject *((*attributeValueFromNsName)(xmlNode (*),char (*),char (*)));
 static PyObject *((*callLookupFallback)(struct LxmlFallbackElementClassLookup *,struct LxmlDocument *,xmlNode (*)));
+static PyObject *((*collectAttributes)(xmlNode (*),int ));
 static struct LxmlElement *((*deepcopyNodeToDocument)(struct LxmlDocument *,xmlNode (*)));
 static int ((*delAttribute)(struct LxmlElement *,PyObject *));
 static int ((*delAttributeFromNsName)(xmlNode (*),char (*),char (*)));
@@ -108,6 +110,7 @@ static struct {char *s; void **p;} _etree_API[] = {
   {"attributeValue", &attributeValue},
   {"attributeValueFromNsName", &attributeValueFromNsName},
   {"callLookupFallback", &callLookupFallback},
+  {"collectAttributes", &collectAttributes},
   {"deepcopyNodeToDocument", &deepcopyNodeToDocument},
   {"delAttribute", &delAttribute},
   {"delAttributeFromNsName", &delAttributeFromNsName},
