@@ -290,6 +290,81 @@ class ETreeTestCaseBase(HelperTestCase):
         self.assertEquals(None, root.get('three'))
         self.assertEquals('foo', root.get('three', 'foo'))
 
+    def test_attrib_clear(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc one="One" two="Two"/>')
+        self.assertEquals('One', root.get('one'))
+        self.assertEquals('Two', root.get('two'))
+        root.attrib.clear()
+        self.assertEquals(None, root.get('one'))
+        self.assertEquals(None, root.get('two'))
+
+    def test_attrib_set_clear(self):
+        Element = self.etree.Element
+        
+        root = Element("root", one="One")
+        root.set("two", "Two")
+        self.assertEquals('One', root.get('one'))
+        self.assertEquals('Two', root.get('two'))
+        root.attrib.clear()
+        self.assertEquals(None, root.get('one'))
+        self.assertEquals(None, root.get('two'))
+
+    def test_attribute_update_dict(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc alpha="Alpha" beta="Beta"/>')
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'Alpha'), ('beta', 'Beta')],
+            items)
+
+        root.attrib.update({'alpha' : 'test', 'gamma' : 'Gamma'})
+
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'test'), ('beta', 'Beta'), ('gamma', 'Gamma')],
+            items)
+
+    def test_attribute_update_sequence(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc alpha="Alpha" beta="Beta"/>')
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'Alpha'), ('beta', 'Beta')],
+            items)
+
+        root.attrib.update({'alpha' : 'test', 'gamma' : 'Gamma'}.items())
+
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'test'), ('beta', 'Beta'), ('gamma', 'Gamma')],
+            items)
+
+    def test_attribute_update_iter(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc alpha="Alpha" beta="Beta"/>')
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'Alpha'), ('beta', 'Beta')],
+            items)
+
+        root.attrib.update({'alpha' : 'test', 'gamma' : 'Gamma'}.iteritems())
+
+        items = root.attrib.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha', 'test'), ('beta', 'Beta'), ('gamma', 'Gamma')],
+            items)
+
     def test_attribute_keys(self):
         XML = self.etree.XML
         
@@ -306,6 +381,16 @@ class ETreeTestCaseBase(HelperTestCase):
         keys.sort()
         self.assertEquals(['alpha', 'beta', 'gamma'], keys)
 
+    def test_attribute_items2(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc alpha="Alpha" beta="Beta" gamma="Gamma"/>')
+        items = root.items()
+        items.sort()
+        self.assertEquals(
+            [('alpha','Alpha'), ('beta','Beta'), ('gamma','Gamma')],
+            items)
+
     def test_attribute_keys_ns(self):
         XML = self.etree.XML
 
@@ -1076,6 +1161,26 @@ class ETreeTestCaseBase(HelperTestCase):
         self.assertXML('<b><bs></bs></b>', b)
         self.assertXML('<c><cs></cs></c>', c)
 
+    def test_delslice_tail(self):
+        XML = self.etree.XML
+        a = XML('<a><b></b>B2<c></c>C2</a>')
+        b, c = a
+
+        del a[:]
+
+        self.assertEquals("B2", b.tail)
+        self.assertEquals("C2", c.tail)
+
+    def test_replace_slice_tail(self):
+        XML = self.etree.XML
+        a = XML('<a><b></b>B2<c></c>C2</a>')
+        b, c = a
+
+        a[:] = []
+
+        self.assertEquals("B2", b.tail)
+        self.assertEquals("C2", c.tail)
+
     def test_delitem_tail(self):
         ElementTree = self.etree.ElementTree
         f = StringIO('<a><b></b>B2<c></c>C2</a>')
@@ -1220,6 +1325,22 @@ class ETreeTestCaseBase(HelperTestCase):
         self.assertXML(
             '<a><c></c></a>',
             a)
+        
+    def test_remove_ns(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('{http://test}a')
+        b = SubElement(a, '{http://test}b')
+        c = SubElement(a, '{http://test}c')
+
+        a.remove(b)
+        self.assertXML(
+            '<ns0:a xmlns:ns0="http://test"><ns0:c></ns0:c></ns0:a>',
+            a)
+        self.assertXML(
+            '<ns0:b xmlns:ns0="http://test"></ns0:b>',
+            b)
 
     def test_remove_nonexisting(self):
         Element = self.etree.Element
@@ -1334,6 +1455,56 @@ class ETreeTestCaseBase(HelperTestCase):
             [a2],
             list(c.getiterator('a')))
 
+    def test_getiterator_filter_comment(self):
+        Element = self.etree.Element
+        Comment = self.etree.Comment
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        comment_b = Comment("TEST-b")
+        b.append(comment_b)
+
+        self.assertEquals(
+            [comment_b],
+            list(a.getiterator(Comment)))
+
+        comment_a = Comment("TEST-a")
+        a.append(comment_a)
+
+        self.assertEquals(
+            [comment_b, comment_a],
+            list(a.getiterator(Comment)))
+
+        self.assertEquals(
+            [comment_b],
+            list(b.getiterator(Comment)))
+
+    def test_getiterator_filter_pi(self):
+        Element = self.etree.Element
+        PI = self.etree.ProcessingInstruction
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        pi_b = PI("TEST-b")
+        b.append(pi_b)
+
+        self.assertEquals(
+            [pi_b],
+            list(a.getiterator(PI)))
+
+        pi_a = PI("TEST-a")
+        a.append(pi_a)
+
+        self.assertEquals(
+            [pi_b, pi_a],
+            list(a.getiterator(PI)))
+
+        self.assertEquals(
+            [pi_b],
+            list(b.getiterator(PI)))
+
     def test_getiterator_with_text(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
