@@ -70,19 +70,38 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals("TEST", root.get("attr"))
         self.assertRaises(TypeError, root.set, "newattr", 5)
 
-    def test_parse_file_dtd(self):
-        parse = self.etree.parse
-        parser = self.etree.XMLParser(attribute_defaults=True)
+    def test_attrib_pop(self):
+        ElementTree = self.etree.ElementTree
+        
+        f = StringIO('<doc one="One" two="Two"/>')
+        doc = ElementTree(file=f)
+        root = doc.getroot()
+        self.assertEquals('One', root.attrib['one'])
+        self.assertEquals('Two', root.attrib['two'])
 
-        tree = parse(fileInTestDir('test.xml'), parser)
-        root = tree.getroot()
+        self.assertEquals('One', root.attrib.pop('one'))
 
-        self.assertEquals(
-            "valueA",
-            root.get("default"))
-        self.assertEquals(
-            "valueB",
-            root[0].get("default"))
+        self.assertEquals(None, root.attrib.get('one'))
+        self.assertEquals('Two', root.attrib['two'])
+
+    def test_attrib_pop_unknown(self):
+        root = self.etree.XML('<doc one="One" two="Two"/>')
+        self.assertRaises(KeyError, root.attrib.pop, 'NONE')
+
+        self.assertEquals('One', root.attrib['one'])
+        self.assertEquals('Two', root.attrib['two'])
+
+    def test_attrib_pop_default(self):
+        root = self.etree.XML('<doc one="One" two="Two"/>')
+        self.assertEquals('Three', root.attrib.pop('three', 'Three'))
+
+    def test_attrib_pop_empty_default(self):
+        root = self.etree.XML('<doc/>')
+        self.assertEquals('Three', root.attrib.pop('three', 'Three'))
+
+    def test_attrib_pop_invalid_args(self):
+        root = self.etree.XML('<doc one="One" two="Two"/>')
+        self.assertRaises(TypeError, root.attrib.pop, 'One', None, None)
 
     def test_pi(self):
         # lxml.etree separates target and text
@@ -142,6 +161,18 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertRaises(SyntaxError, parse, f)
         f.close()
 
+    def test_parse_remove_comments(self):
+        parse = self.etree.parse
+        tostring = self.etree.tostring
+        XMLParser = self.etree.XMLParser
+
+        f = StringIO('<a><!--A--><b><!-- B --><c/></b><!--C--></a>')
+        parser = XMLParser(remove_comments=True)
+        tree = parse(f, parser)
+        self.assertEquals(
+            '<a><b><c/></b></a>',
+            tostring(tree))
+
     def test_parse_parser_type_error(self):
         # ET raises IOError only
         parse = self.etree.parse
@@ -172,22 +203,36 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertRaises(SyntaxError, parse, f)
         f.close()
 
+    def test_iterparse_comments(self):
+        # ET removes comments
+        iterparse = self.etree.iterparse
+        tostring = self.etree.tostring
+
+        f = StringIO('<a><!--A--><b><!-- B --><c/></b><!--C--></a>')
+        events = list(iterparse(f))
+        root = events[-1][1]
+        self.assertEquals(3, len(events))
+        self.assertEquals(
+            '<a><!--A--><b><!-- B --><c/></b><!--C--></a>',
+            tostring(root))
+
+    def test_iterparse_remove_comments(self):
+        iterparse = self.etree.iterparse
+        tostring = self.etree.tostring
+
+        f = StringIO('<a><!--A--><b><!-- B --><c/></b><!--C--></a>')
+        events = list(iterparse(f, remove_comments=True))
+        root = events[-1][1]
+        self.assertEquals(
+            '<a><b><c/></b></a>',
+            tostring(root))
+
     def test_iterparse_broken(self):
         iterparse = self.etree.iterparse
         f = StringIO('<a><b><c/></a>')
         # ET raises ExpatError, lxml raises XMLSyntaxError
         self.assertRaises(self.etree.XMLSyntaxError, list, iterparse(f))
 
-    def test_iterparse_file_dtd(self):
-        iterparse = self.etree.iterparse
-        iterator = iterparse(fileInTestDir("test.xml"), events=("start",),
-                             attribute_defaults=True)
-        attributes = [ element.get("default")
-                       for event, element in iterator ]
-        self.assertEquals(
-            ["valueA", "valueB"],
-            attributes)
-
     def test_iterparse_strip(self):
         iterparse = self.etree.iterparse
         f = StringIO("""
@@ -395,6 +440,165 @@ class ETreeOnlyTestCase(HelperTestCase):
         Element = self.etree.Element
         self.assertRaises(TypeError, Element('a').append, None)
 
+    def test_addnext(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        root = Element('root')
+        SubElement(root, 'a')
+        SubElement(root, 'b')
+
+        self.assertEquals(['a', 'b'],
+                          [c.tag for c in root])
+        root[1].addnext(root[0])
+        self.assertEquals(['b', 'a'],
+                          [c.tag for c in root])
+
+    def test_addprevious(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        root = Element('root')
+        SubElement(root, 'a')
+        SubElement(root, 'b')
+
+        self.assertEquals(['a', 'b'],
+                          [c.tag for c in root])
+        root[0].addprevious(root[1])
+        self.assertEquals(['b', 'a'],
+                          [c.tag for c in root])
+
+    def test_addnext_root(self):
+        Element = self.etree.Element
+        a = Element('a')
+        b = Element('b')
+        self.assertRaises(TypeError, a.addnext, b)
+
+    def test_addnext_root(self):
+        Element = self.etree.Element
+        a = Element('a')
+        b = Element('b')
+        self.assertRaises(TypeError, a.addnext, b)
+
+    def test_addprevious_pi(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        PI = self.etree.PI
+        root = Element('root')
+        SubElement(root, 'a')
+        pi = PI('TARGET', 'TEXT')
+        pi.tail = "TAIL"
+
+        self.assertEquals('<root><a></a></root>',
+                          self._writeElement(root))
+        root[0].addprevious(pi)
+        self.assertEquals('<root><?TARGET TEXT?>TAIL<a></a></root>',
+                          self._writeElement(root))
+
+    def test_addprevious_root_pi(self):
+        Element = self.etree.Element
+        PI = self.etree.PI
+        root = Element('root')
+        pi = PI('TARGET', 'TEXT')
+        pi.tail = "TAIL"
+
+        self.assertEquals('<root></root>',
+                          self._writeElement(root))
+        root.addprevious(pi)
+        self.assertEquals('<?TARGET TEXT?>\n<root></root>',
+                          self._writeElement(root))
+
+    def test_addnext_pi(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        PI = self.etree.PI
+        root = Element('root')
+        SubElement(root, 'a')
+        pi = PI('TARGET', 'TEXT')
+        pi.tail = "TAIL"
+
+        self.assertEquals('<root><a></a></root>',
+                          self._writeElement(root))
+        root[0].addnext(pi)
+        self.assertEquals('<root><a></a><?TARGET TEXT?>TAIL</root>',
+                          self._writeElement(root))
+
+    def test_addnext_root_pi(self):
+        Element = self.etree.Element
+        PI = self.etree.PI
+        root = Element('root')
+        pi = PI('TARGET', 'TEXT')
+        pi.tail = "TAIL"
+
+        self.assertEquals('<root></root>',
+                          self._writeElement(root))
+        root.addnext(pi)
+        self.assertEquals('<root></root>\n<?TARGET TEXT?>',
+                          self._writeElement(root))
+
+    def test_addnext_comment(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        Comment = self.etree.Comment
+        root = Element('root')
+        SubElement(root, 'a')
+        comment = Comment('TEXT ')
+        comment.tail = "TAIL"
+
+        self.assertEquals('<root><a></a></root>',
+                          self._writeElement(root))
+        root[0].addnext(comment)
+        self.assertEquals('<root><a></a><!--TEXT -->TAIL</root>',
+                          self._writeElement(root))
+
+    def test_addnext_root_comment(self):
+        Element = self.etree.Element
+        Comment = self.etree.Comment
+        root = Element('root')
+        comment = Comment('TEXT ')
+        comment.tail = "TAIL"
+
+        self.assertEquals('<root></root>',
+                          self._writeElement(root))
+        root.addnext(comment)
+        self.assertEquals('<root></root>\n<!--TEXT -->',
+                          self._writeElement(root))
+
+    def test_addprevious_comment(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        Comment = self.etree.Comment
+        root = Element('root')
+        SubElement(root, 'a')
+        comment = Comment('TEXT ')
+        comment.tail = "TAIL"
+
+        self.assertEquals('<root><a></a></root>',
+                          self._writeElement(root))
+        root[0].addprevious(comment)
+        self.assertEquals('<root><!--TEXT -->TAIL<a></a></root>',
+                          self._writeElement(root))
+
+    def test_addprevious_root_comment(self):
+        Element = self.etree.Element
+        Comment = self.etree.Comment
+        root = Element('root')
+        comment = Comment('TEXT ')
+        comment.tail = "TAIL"
+
+        self.assertEquals('<root></root>',
+                          self._writeElement(root))
+        root.addprevious(comment)
+        self.assertEquals('<!--TEXT -->\n<root></root>',
+                          self._writeElement(root))
+
+    # ET's Elements have items() and key(), but not values()
+    def test_attribute_values(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc alpha="Alpha" beta="Beta" gamma="Gamma"/>')
+        values = root.values()
+        values.sort()
+        self.assertEquals(['Alpha', 'Beta', 'Gamma'], values)
+
     # gives error in ElementTree
     def test_comment_empty(self):
         Element = self.etree.Element
@@ -893,6 +1097,81 @@ class ETreeOnlyTestCase(HelperTestCase):
             '<z xmlns="http://ns.infrae.com/foo" xmlns:hoi="http://ns.infrae.com/hoi"><hoi:x></hoi:x></z>',
             self._writeElement(e))
 
+    def test_namespaces_default_copy_element(self):
+        etree = self.etree
+
+        r = {None: 'http://ns.infrae.com/foo'}
+        e1 = etree.Element('{http://ns.infrae.com/foo}bar', nsmap=r)
+        e2 = etree.Element('{http://ns.infrae.com/foo}bar', nsmap=r)
+
+        e1.append(e2)
+
+        self.assertEquals(
+            None,
+            e1.prefix)
+        self.assertEquals(
+            None,
+            e1[0].prefix)
+        self.assertEquals(
+            '{http://ns.infrae.com/foo}bar',
+            e1.tag)
+        self.assertEquals(
+            '{http://ns.infrae.com/foo}bar',
+            e1[0].tag)
+
+    def test_namespaces_copy_element(self):
+        etree = self.etree
+
+        r = {None: 'http://ns.infrae.com/BAR'}
+        e1 = etree.Element('{http://ns.infrae.com/BAR}bar', nsmap=r)
+        e2 = etree.Element('{http://ns.infrae.com/foo}bar', nsmap=r)
+
+        e1.append(e2)
+
+        self.assertEquals(
+            None,
+            e1.prefix)
+        self.assertNotEquals(
+            None,
+            e2.prefix)
+        self.assertEquals(
+            '{http://ns.infrae.com/BAR}bar',
+            e1.tag)
+        self.assertEquals(
+            '{http://ns.infrae.com/foo}bar',
+            e2.tag)
+
+    def test_namespaces_reuse_after_move(self):
+        ns_href = "http://a.b.c"
+        one = self.etree.parse(
+            StringIO('<foo><bar xmlns:ns="%s"><ns:baz/></bar></foo>' % ns_href))
+        baz = one.getroot()[0][0]
+
+        two = self.etree.parse(
+            StringIO('<root xmlns:ns="%s"/>' % ns_href))
+        two.getroot().append(baz)
+        del one # make sure the source document is deallocated
+
+        self.assertEquals('{%s}baz' % ns_href, baz.tag)
+        self.assertEquals(
+            '<root xmlns:ns="%s"><ns:baz/></root>' % ns_href,
+            self.etree.tostring(two))
+
+    def _test_namespaces_after_serialize(self):
+        # FIXME: this currently fails - fix serializer.pxi!
+        parse = self.etree.parse
+        tostring = self.etree.tostring
+
+        ns_href = "http://a.b.c"
+        one = parse(
+            StringIO('<foo><bar xmlns:ns="%s"><ns:baz/></bar></foo>' % ns_href))
+        baz = one.getroot()[0][0]
+
+        print tostring(baz)
+        parsed = parse(StringIO( tostring(baz) )).getroot()
+
+        self.assertEquals('{%s}baz' % ns_href, parsed.tag)
+
     def test_element_nsmap(self):
         etree = self.etree
 
@@ -1068,6 +1347,37 @@ class ETreeOnlyTestCase(HelperTestCase):
             ["tail0", "tail1", "tail2", "TAIL0", "TAIL1", "TAIL2"],
             [ el.tail for el in root ])
 
+    def test_sourceline_XML(self):
+        XML = self.etree.XML
+        root = XML('''<?xml version="1.0"?>
+        <root><test>
+
+        <bla/></test>
+        </root>
+        ''')
+
+        self.assertEquals(
+            [2, 2, 4],
+            [ el.sourceline for el in root.getiterator() ])
+
+    def test_sourceline_parse(self):
+        parse = self.etree.parse
+        tree = parse(fileInTestDir('include/test_xinclude.xml'))
+
+        self.assertEquals(
+            [1, 2, 3],
+            [ el.sourceline for el in tree.getiterator() ])
+
+    def test_sourceline_element(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        el = Element("test")
+        self.assertEquals(None, el.sourceline)
+
+        child = SubElement(el, "test")
+        self.assertEquals(None, el.sourceline)
+        self.assertEquals(None, child.sourceline)
+
     def test_docinfo_public(self):
         etree = self.etree
         xml_header = '<?xml version="1.0" encoding="ascii"?>'
@@ -1114,6 +1424,50 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(docinfo.root_name,   'html')
         self.assertEquals(docinfo.doctype, '')
 
+    def test_byte_zero(self):
+        Element = self.etree.Element
+
+        a = Element('a')
+        self.assertRaises(AssertionError, setattr, a, "text", 'ha\0ho')
+        self.assertRaises(AssertionError, setattr, a, "tail", 'ha\0ho')
+
+        self.assertRaises(AssertionError, Element, 'ha\0ho')
+
+    def test_unicode_byte_zero(self):
+        Element = self.etree.Element
+
+        a = Element('a')
+        self.assertRaises(AssertionError, setattr, a, "text", u'ha\0ho')
+        self.assertRaises(AssertionError, setattr, a, "tail", u'ha\0ho')
+
+        self.assertRaises(AssertionError, Element, u'ha\0ho')
+
+    def test_byte_invalid(self):
+        Element = self.etree.Element
+
+        a = Element('a')
+        self.assertRaises(AssertionError, setattr, a, "text", 'ha\x07ho')
+        self.assertRaises(AssertionError, setattr, a, "text", 'ha\x02ho')
+
+        self.assertRaises(AssertionError, setattr, a, "tail", 'ha\x07ho')
+        self.assertRaises(AssertionError, setattr, a, "tail", 'ha\x02ho')
+
+        self.assertRaises(AssertionError, Element, 'ha\x07ho')
+        self.assertRaises(AssertionError, Element, 'ha\x02ho')
+
+    def test_unicode_byte_invalid(self):
+        Element = self.etree.Element
+
+        a = Element('a')
+        self.assertRaises(AssertionError, setattr, a, "text", u'ha\x07ho')
+        self.assertRaises(AssertionError, setattr, a, "text", u'ha\x02ho')
+
+        self.assertRaises(AssertionError, setattr, a, "tail", u'ha\x07ho')
+        self.assertRaises(AssertionError, setattr, a, "tail", u'ha\x02ho')
+
+        self.assertRaises(AssertionError, Element, u'ha\x07ho')
+        self.assertRaises(AssertionError, Element, u'ha\x02ho')
+
     def test_encoding_tostring_utf16(self):
         # ElementTree fails to serialize this
         tostring = self.etree.tostring
@@ -1244,12 +1598,8 @@ class XIncludeTestCase(HelperTestCase):
         self.assertEquals(old_text + content + old_tail,
                           root.text)
 
-class ETreeXIncludeTestCase(XIncludeTestCase):
-    def include(self, tree):
-        tree.xinclude()
-
     def test_xinclude(self):
-        tree = etree.parse(fileInTestDir('test_xinclude.xml'))
+        tree = etree.parse(fileInTestDir('include/test_xinclude.xml'))
         # process xincludes
         self.include( tree )
         # check whether we find it replaced with included data
@@ -1257,6 +1607,10 @@ class ETreeXIncludeTestCase(XIncludeTestCase):
             'a',
             tree.getroot()[1].tag)
 
+class ETreeXIncludeTestCase(XIncludeTestCase):
+    def include(self, tree):
+        tree.xinclude()
+
 
 class ElementIncludeTestCase(XIncludeTestCase):
     from lxml import ElementInclude
@@ -1279,8 +1633,12 @@ def test_suite():
     suite.addTests([unittest.makeSuite(ETreeXIncludeTestCase)])
     suite.addTests([unittest.makeSuite(ElementIncludeTestCase)])
     suite.addTests([unittest.makeSuite(ETreeC14NTestCase)])
+    suite.addTests(
+        [doctest.DocFileSuite('../../../doc/tutorial.txt')])
     suite.addTests(
         [doctest.DocFileSuite('../../../doc/api.txt')])
+    suite.addTests(
+        [doctest.DocFileSuite('../../../doc/parsing.txt')])
     suite.addTests(
         [doctest.DocFileSuite('../../../doc/resolvers.txt')])
     return suite
