@@ -1,10 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Tests specific to the extended etree API
-
-Tests that apply to the general ElementTree API should go into
-test_elementtree
+Tests specific to the lxml.objectify API
 """
 
 
@@ -16,6 +13,33 @@ from common_imports import itemgetter
 
 from lxml import objectify
 
+PYTYPE_NAMESPACE = "http://codespeak.net/lxml/objectify/pytype"
+XML_SCHEMA_NS = "http://www.w3.org/2001/XMLSchema"
+XML_SCHEMA_INSTANCE_NS = "http://www.w3.org/2001/XMLSchema-instance"
+XML_SCHEMA_INSTANCE_TYPE_ATTR = "{%s}type" % XML_SCHEMA_INSTANCE_NS
+XML_SCHEMA_NIL_ATTR = "{%s}nil" % XML_SCHEMA_INSTANCE_NS
+DEFAULT_NSMAP = { "py" : PYTYPE_NAMESPACE,
+                  "xsi" : XML_SCHEMA_INSTANCE_NS,
+                  "xsd" : XML_SCHEMA_NS}
+
+objectclass2xsitype = {
+    # objectify built-in
+    objectify.IntElement: ("int", "short", "byte", "unsignedShort",
+                           "unsignedByte",),
+    objectify.LongElement: ("integer", "nonPositiveInteger", "negativeInteger",
+                            "long", "nonNegativeInteger", "unsignedLong",
+                            "unsignedInt", "positiveInteger",),
+    objectify.FloatElement: ("float", "double"),
+    objectify.BoolElement: ("boolean",),
+    objectify.StringElement: ("string", "normalizedString", "token", "language",
+                              "Name", "NCName", "ID", "IDREF", "ENTITY",
+                              "NMTOKEN", ),
+    # None: xsi:nil="true"
+    }
+
+xsitype2objclass = dict(( (v, k) for k in objectclass2xsitype
+                          for v in objectclass2xsitype[k] ))
+
 xml_str = '''\
 <obj:root xmlns:obj="objectified" xmlns:other="otherNS">
   <obj:c1 a1="A1" a2="A2" other:a3="A3">
@@ -28,10 +52,10 @@ xml_str = '''\
 </obj:root>'''
 
 class ObjectifyTestCase(HelperTestCase):
-    """Test cases for lxml.elementlib.objectify
+    """Test cases for lxml.objectify
     """
     etree = etree
-
+    
     def XML(self, xml):
         return self.etree.XML(xml, self.parser)
 
@@ -50,6 +74,116 @@ class ObjectifyTestCase(HelperTestCase):
         self.etree.Namespace("otherNS").clear()
         objectify.setPytypeAttributeTag()
 
+    def test_element_nsmap(self):
+        # default nsmap
+        root = objectify.Element("test")
+        self.assertEquals(root.nsmap, DEFAULT_NSMAP)
+
+        # empty nsmap
+        nsmap = {}
+        root = objectify.Element("test", nsmap=nsmap)
+        self.assertEquals(root.nsmap.values(), [PYTYPE_NAMESPACE])
+
+        # nsmap with custom prefixes
+        nsmap = {"mypy" : PYTYPE_NAMESPACE,
+                 "myxsi" : XML_SCHEMA_INSTANCE_NS,
+                 "myxsd" : XML_SCHEMA_NS}
+        root = objectify.Element("test", nsmap=nsmap)
+        self.assertEquals(root.nsmap, nsmap)
+        
+        # custom nsmap
+        nsmap = {"my"  : "someNS",
+                 "myother" : "someOtherNS",
+                 }
+        root = objectify.Element("test", nsmap=nsmap)
+        self.assert_(PYTYPE_NAMESPACE in root.nsmap.values())
+        for prefix, ns in nsmap.items():
+            self.assert_(prefix in root.nsmap)
+            self.assertEquals(nsmap[prefix], root.nsmap[prefix]) 
+        
+    def test_sub_element_nsmap(self):
+        root = objectify.Element("root")
+        # default nsmap
+        root.sub = objectify.Element("test")
+        self.assertEquals(root.sub.nsmap, DEFAULT_NSMAP)
+
+        # empty nsmap
+        nsmap = {}
+        root.sub = objectify.Element("test", nsmap=nsmap)
+        self.assertEquals(root.sub.nsmap, DEFAULT_NSMAP)
+
+        # nsmap with custom prefixes
+        nsmap = {"mypy" : PYTYPE_NAMESPACE,
+                 "myxsi" : XML_SCHEMA_INSTANCE_NS,
+                 "myxsd" : XML_SCHEMA_NS}
+        root.sub = objectify.Element("test", nsmap=nsmap)
+        self.assertEquals(root.sub.nsmap, DEFAULT_NSMAP)
+        
+        # custom nsmap
+        nsmap = {"my"  : "someNS",
+                 "myother" : "someOtherNS",
+                 }
+        root.sub = objectify.Element("test", nsmap=nsmap)
+        expected = nsmap.copy()
+        expected.update(DEFAULT_NSMAP)
+        self.assertEquals(root.sub.nsmap, expected) 
+        
+    def test_data_element_nsmap(self):
+        # default nsmap
+        value = objectify.DataElement("test this")
+        self.assertEquals(value.nsmap, DEFAULT_NSMAP)
+
+        # empty nsmap
+        nsmap = {}
+        value = objectify.DataElement("test this", nsmap=nsmap)
+        self.assertEquals(value.nsmap.values(), [PYTYPE_NAMESPACE])
+
+        # nsmap with custom prefixes
+        nsmap = {"mypy" : PYTYPE_NAMESPACE,
+                 "myxsi" : XML_SCHEMA_INSTANCE_NS,
+                 "myxsd" : XML_SCHEMA_NS}
+
+        value = objectify.DataElement("test this", nsmap=nsmap)
+        self.assertEquals(value.nsmap, nsmap)
+        
+        # custom nsmap
+        nsmap = {"my"  : "someNS",
+                 "myother" : "someOtherNS",
+                 }
+        value = objectify.DataElement("test", nsmap=nsmap)
+        self.assert_(PYTYPE_NAMESPACE in value.nsmap.values())
+        for prefix, ns in nsmap.items():
+            self.assert_(prefix in value.nsmap)
+            self.assertEquals(nsmap[prefix], value.nsmap[prefix]) 
+        
+    def test_sub_data_element_nsmap(self):
+        root = objectify.Element("root")
+        # default nsmap
+        root.value = objectify.DataElement("test this")
+        self.assertEquals(root.value.nsmap, DEFAULT_NSMAP)
+
+        # empty nsmap
+        nsmap = {}
+        root.value = objectify.DataElement("test this", nsmap=nsmap)
+        self.assertEquals(root.value.nsmap, DEFAULT_NSMAP)
+
+        # nsmap with custom prefixes
+        nsmap = {"mypy" : PYTYPE_NAMESPACE,
+                 "myxsi" : XML_SCHEMA_INSTANCE_NS,
+                 "myxsd" : XML_SCHEMA_NS}
+
+        root.value = objectify.DataElement("test this", nsmap=nsmap)
+        self.assertEquals(root.value.nsmap, DEFAULT_NSMAP)
+        
+        # custom nsmap
+        nsmap = {"my"  : "someNS",
+                 "myother" : "someOtherNS",
+                 }
+        root.value = objectify.DataElement("test", nsmap=nsmap)
+        expected = nsmap.copy()
+        expected.update(DEFAULT_NSMAP)
+        self.assertEquals(root.value.nsmap, expected) 
+        
     def test_root(self):
         root = self.Element("test")
         self.assert_(isinstance(root, objectify.ObjectifiedElement))
@@ -285,8 +419,13 @@ class ObjectifyTestCase(HelperTestCase):
         Element = self.Element
         SubElement = self.etree.SubElement
         root = Element("{objectified}root")
-        root.none = 'true'
-        self.assert_(isinstance(root.none, objectify.BoolElement))
+        root.bool = 'true'
+        self.assert_(isinstance(root.bool, objectify.BoolElement))
+        self.assertEquals(root.bool, True)
+
+        root.bool = 'false'
+        self.assert_(isinstance(root.bool, objectify.BoolElement))
+        self.assertEquals(root.bool, False)
 
     def test_data_element_bool(self):
         value = objectify.DataElement(True)
@@ -355,25 +494,161 @@ class ObjectifyTestCase(HelperTestCase):
         self.assert_(isinstance(value, objectify.FloatElement))
         self.assertEquals(value, 5.5)
 
+    def test_data_element_xsitypes(self):
+        for xsi, objclass in xsitype2objclass.iteritems():
+            # 1 is a valid value for all ObjectifiedDataElement classes
+            value = objectify.DataElement(1, _xsi=xsi)
+            self.assert_(isinstance(value, objclass))
+        
+    def test_data_element_xsitypes_xsdprefixed(self):
+        for xsi, objclass in xsitype2objclass.iteritems():
+            # 1 is a valid value for all ObjectifiedDataElement classes
+            value = objectify.DataElement(1, _xsi="xsd:%s" % xsi)
+            self.assert_(isinstance(value, objclass))
+        
+    def test_data_element_xsitypes_prefixed(self):
+        for xsi, objclass in xsitype2objclass.iteritems():
+            # 1 is a valid value for all ObjectifiedDataElement classes
+            self.assertRaises(ValueError, objectify.DataElement, 1,
+                              _xsi="foo:%s" % xsi)
+
     def test_schema_types(self):
         XML = self.XML
         root = XML('''\
         <root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
-          <a xsi:type="integer">5</a>
-          <a xsi:type="string">5</a>
-          <a xsi:type="float">5</a>
+          <b xsi:type="boolean">true</b>
+          <b xsi:type="boolean">false</b>
+          <b xsi:type="boolean">1</b>
+          <b xsi:type="boolean">0</b>
+
+          <f xsi:type="float">5</f>
+          <f xsi:type="double">5</f>
+        
+          <s xsi:type="string">5</s>
+          <s xsi:type="normalizedString">5</s>
+          <s xsi:type="token">5</s>
+          <s xsi:type="language">5</s>
+          <s xsi:type="Name">5</s>
+          <s xsi:type="NCName">5</s>
+          <s xsi:type="ID">5</s>
+          <s xsi:type="IDREF">5</s>
+          <s xsi:type="ENTITY">5</s>
+          <s xsi:type="NMTOKEN">5</s>
+
+          <l xsi:type="integer">5</l>
+          <l xsi:type="nonPositiveInteger">5</l>
+          <l xsi:type="negativeInteger">5</l>
+          <l xsi:type="long">5</l>
+          <l xsi:type="nonNegativeInteger">5</l>
+          <l xsi:type="unsignedLong">5</l>
+          <l xsi:type="unsignedInt">5</l>
+          <l xsi:type="positiveInteger">5</l>
+          
+          <i xsi:type="int">5</i>
+          <i xsi:type="short">5</i>
+          <i xsi:type="byte">5</i>
+          <i xsi:type="unsignedShort">5</i>
+          <i xsi:type="unsignedByte">5</i>
+
+          <n xsi:nil="true"/>
         </root>
         ''')
 
-        self.assert_(isinstance(root.a[0], objectify.IntElement))
-        self.assertEquals(5, root.a[0])
+        for b in root.b:
+            self.assert_(isinstance(b, objectify.BoolElement))
+        self.assertEquals(True,  root.b[0])
+        self.assertEquals(False, root.b[1])
+        self.assertEquals(True,  root.b[2])
+        self.assertEquals(False, root.b[3])
 
-        self.assert_(isinstance(root.a[1], objectify.StringElement))
-        self.assertEquals("5", root.a[1])
+        for f in root.f:
+            self.assert_(isinstance(f, objectify.FloatElement))
+            self.assertEquals(5, f)
+            
+        for s in root.s:
+            self.assert_(isinstance(s, objectify.StringElement))
+            self.assertEquals("5", s)
 
-        self.assert_(isinstance(root.a[2], objectify.FloatElement))
-        self.assertEquals(5.0, root.a[2])
+        for l in root.l:
+            self.assert_(isinstance(l, objectify.LongElement))
+            self.assertEquals(5L, l)
 
+        for i in root.i:
+            self.assert_(isinstance(i, objectify.IntElement))
+            self.assertEquals(5, i)
+            
+        self.assert_(isinstance(root.n, objectify.NoneElement))
+        self.assertEquals(None, root.n)
+
+    def test_schema_types_prefixed(self):
+        XML = self.XML
+        root = XML('''\
+        <root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+        xmlns:xsd="http://www.w3.org/2001/XMLSchema">
+          <b xsi:type="xsd:boolean">true</b>
+          <b xsi:type="xsd:boolean">false</b>
+          <b xsi:type="xsd:boolean">1</b>
+          <b xsi:type="xsd:boolean">0</b>
+
+          <f xsi:type="xsd:float">5</f>
+          <f xsi:type="xsd:double">5</f>
+        
+          <s xsi:type="xsd:string">5</s>
+          <s xsi:type="xsd:normalizedString">5</s>
+          <s xsi:type="xsd:token">5</s>
+          <s xsi:type="xsd:language">5</s>
+          <s xsi:type="xsd:Name">5</s>
+          <s xsi:type="xsd:NCName">5</s>
+          <s xsi:type="xsd:ID">5</s>
+          <s xsi:type="xsd:IDREF">5</s>
+          <s xsi:type="xsd:ENTITY">5</s>
+          <s xsi:type="xsd:NMTOKEN">5</s>
+
+          <l xsi:type="xsd:integer">5</l>
+          <l xsi:type="xsd:nonPositiveInteger">5</l>
+          <l xsi:type="xsd:negativeInteger">5</l>
+          <l xsi:type="xsd:long">5</l>
+          <l xsi:type="xsd:nonNegativeInteger">5</l>
+          <l xsi:type="xsd:unsignedLong">5</l>
+          <l xsi:type="xsd:unsignedInt">5</l>
+          <l xsi:type="xsd:positiveInteger">5</l>
+          
+          <i xsi:type="xsd:int">5</i>
+          <i xsi:type="xsd:short">5</i>
+          <i xsi:type="xsd:byte">5</i>
+          <i xsi:type="xsd:unsignedShort">5</i>
+          <i xsi:type="xsd:unsignedByte">5</i>
+
+          <n xsi:nil="true"/>
+        </root>
+        ''')
+
+        for b in root.b:
+            self.assert_(isinstance(b, objectify.BoolElement))
+        self.assertEquals(True,  root.b[0])
+        self.assertEquals(False, root.b[1])
+        self.assertEquals(True,  root.b[2])
+        self.assertEquals(False, root.b[3])
+
+        for f in root.f:
+            self.assert_(isinstance(f, objectify.FloatElement))
+            self.assertEquals(5, f)
+            
+        for s in root.s:
+            self.assert_(isinstance(s, objectify.StringElement))
+            self.assertEquals("5", s)
+
+        for l in root.l:
+            self.assert_(isinstance(l, objectify.LongElement))
+            self.assertEquals(5L, l)
+
+        for i in root.i:
+            self.assert_(isinstance(i, objectify.IntElement))
+            self.assertEquals(5, i)
+            
+        self.assert_(isinstance(root.n, objectify.NoneElement))
+        self.assertEquals(None, root.n)
+        
     def test_type_str_sequence(self):
         XML = self.XML
         root = XML(u'<root><b>why</b><b>try</b></root>')
@@ -447,10 +722,11 @@ class ObjectifyTestCase(HelperTestCase):
         root.b = False
         self.assertFalse(root.b)
 
-    def test_type_annotation(self):
+    def test_pytype_annotation(self):
         XML = self.XML
         root = XML(u'''\
-        <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
+        <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+        xmlns:py="http://codespeak.net/lxml/objectify/pytype">
           <b>5</b>
           <b>test</b>
           <c>1.1</c>
@@ -459,20 +735,190 @@ class ObjectifyTestCase(HelperTestCase):
           <n xsi:nil="true" />
           <n></n>
           <b xsi:type="double">5</b>
+          <b xsi:type="float">5</b>
+          <s xsi:type="string">23</s>
+          <s py:pytype="str">42</s>
+          <f py:pytype="float">300</f>
+          <l py:pytype="long">2</l>
         </a>
         ''')
         objectify.annotate(root)
 
         child_types = [ c.get(objectify.PYTYPE_ATTRIBUTE)
                         for c in root.iterchildren() ]
-        self.assertEquals("int",   child_types[0])
-        self.assertEquals("str",   child_types[1])
-        self.assertEquals("float", child_types[2])
-        self.assertEquals("str",   child_types[3])
-        self.assertEquals("bool",  child_types[4])
-        self.assertEquals("none",  child_types[5])
-        self.assertEquals(None,    child_types[6])
-        self.assertEquals("float", child_types[7])
+        self.assertEquals("int",   child_types[ 0])
+        self.assertEquals("str",   child_types[ 1])
+        self.assertEquals("float", child_types[ 2])
+        self.assertEquals("str",   child_types[ 3])
+        self.assertEquals("bool",  child_types[ 4])
+        self.assertEquals("none",  child_types[ 5])
+        self.assertEquals(None,    child_types[ 6])
+        self.assertEquals("float", child_types[ 7])
+        self.assertEquals("float", child_types[ 8])
+        self.assertEquals("str",   child_types[ 9])
+        self.assertEquals("int",   child_types[10])
+        self.assertEquals("int",   child_types[11])
+        self.assertEquals("int",   child_types[12])
+        
+        self.assertEquals("true", root.n.get(XML_SCHEMA_NIL_ATTR))
+
+    def test_pytype_annotation_use_old(self):
+        XML = self.XML
+        root = XML(u'''\
+        <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+        xmlns:py="http://codespeak.net/lxml/objectify/pytype">
+          <b>5</b>
+          <b>test</b>
+          <c>1.1</c>
+          <c>\uF8D2</c>
+          <x>true</x>
+          <n xsi:nil="true" />
+          <n></n>
+          <b xsi:type="double">5</b>
+          <b xsi:type="float">5</b>
+          <s xsi:type="string">23</s>
+          <s py:pytype="str">42</s>
+          <f py:pytype="float">300</f>
+          <l py:pytype="long">2</l>
+        </a>
+        ''')
+        objectify.annotate(root, ignore_old=False)
+
+        child_types = [ c.get(objectify.PYTYPE_ATTRIBUTE)
+                        for c in root.iterchildren() ]
+        self.assertEquals("int",   child_types[ 0])
+        self.assertEquals("str",   child_types[ 1])
+        self.assertEquals("float", child_types[ 2])
+        self.assertEquals("str",   child_types[ 3])
+        self.assertEquals("bool",  child_types[ 4])
+        self.assertEquals("none",  child_types[ 5])
+        self.assertEquals(None,    child_types[ 6])
+        self.assertEquals("float", child_types[ 7])
+        self.assertEquals("float", child_types[ 8])
+        self.assertEquals("str",   child_types[ 9])
+        self.assertEquals("str",   child_types[10])
+        self.assertEquals("float", child_types[11])
+        self.assertEquals("long",  child_types[12])
+        
+        self.assertEquals("true", root.n.get(XML_SCHEMA_NIL_ATTR))
+
+    def test_deannotate(self):
+        XML = self.XML
+        root = XML(u'''\
+        <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+        xmlns:py="http://codespeak.net/lxml/objectify/pytype">
+          <b>5</b>
+          <b>test</b>
+          <c>1.1</c>
+          <c>\uF8D2</c>
+          <x>true</x>
+          <n xsi:nil="true" />
+          <n></n>
+          <b xsi:type="double">5</b>
+          <b xsi:type="float">5</b>
+          <s xsi:type="string">23</s>
+          <s py:pytype="str">42</s>
+          <f py:pytype="float">300</f>
+          <l py:pytype="long">2</l>
+        </a>
+        ''')
+        objectify.deannotate(root)
+
+        for c in root.getiterator():
+            self.assertEquals(None, c.get(XML_SCHEMA_INSTANCE_TYPE_ATTR))
+            self.assertEquals(None, c.get(objectify.PYTYPE_ATTRIBUTE))
+
+        self.assertEquals("true", root.n.get(XML_SCHEMA_NIL_ATTR))
+
+    def test_xsitype_deannotate(self):
+        XML = self.XML
+        root = XML(u'''\
+        <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+        xmlns:py="http://codespeak.net/lxml/objectify/pytype"
+        xmlns:xsd="http://www.w3.org/2001/XMLSchema">
+          <b>5</b>
+          <b>test</b>
+          <c>1.1</c>
+          <c>\uF8D2</c>
+          <x>true</x>
+          <n xsi:nil="true" />
+          <n></n>
+          <b xsi:type="xsd:double">5</b>
+          <b xsi:type="xsd:float">5</b>
+          <s xsi:type="xsd:string">23</s>
+          <s py:pytype="str">42</s>
+          <f py:pytype="float">300</f>
+          <l py:pytype="long">2</l>
+        </a>
+        ''')
+        objectify.annotate(root)
+        objectify.deannotate(root, pytype=False)
+
+        child_types = [ c.get(objectify.PYTYPE_ATTRIBUTE)
+                        for c in root.iterchildren() ]
+        self.assertEquals("int",   child_types[ 0])
+        self.assertEquals("str",   child_types[ 1])
+        self.assertEquals("float", child_types[ 2])
+        self.assertEquals("str",   child_types[ 3])
+        self.assertEquals("bool",  child_types[ 4])
+        self.assertEquals("none",  child_types[ 5])
+        self.assertEquals(None,    child_types[ 6])
+        self.assertEquals("float", child_types[ 7])
+        self.assertEquals("float", child_types[ 8])
+        self.assertEquals("str",   child_types[ 9])
+        self.assertEquals("int",   child_types[10])
+        self.assertEquals("int",   child_types[11])
+        self.assertEquals("int",   child_types[12])
+        
+        self.assertEquals("true", root.n.get(XML_SCHEMA_NIL_ATTR))
+
+        for c in root.getiterator():
+            self.assertEquals(None, c.get(XML_SCHEMA_INSTANCE_TYPE_ATTR))
+
+    def test_pytype_deannotate(self):
+        XML = self.XML
+        root = XML(u'''\
+        <a xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+        xmlns:py="http://codespeak.net/lxml/objectify/pytype"
+        xmlns:xsd="http://www.w3.org/2001/XMLSchema">
+          <b xsi:type="xsd:int">5</b>
+          <b xsi:type="xsd:string">test</b>
+          <c xsi:type="xsd:float">1.1</c>
+          <c xsi:type="xsd:string">\uF8D2</c>
+          <x xsi:type="xsd:boolean">true</x>
+          <n xsi:nil="true" />
+          <n></n>
+          <b xsi:type="xsd:double">5</b>
+          <b xsi:type="xsd:float">5</b>
+          <s xsi:type="xsd:string">23</s>
+          <s xsi:type="xsd:string">42</s>
+          <f xsi:type="xsd:float">300</f>
+          <l xsi:type="xsd:long">2</l>
+        </a>
+        ''')
+        objectify.annotate(root)
+        objectify.deannotate(root, xsi=False)
+
+        child_types = [ c.get(XML_SCHEMA_INSTANCE_TYPE_ATTR)
+                        for c in root.iterchildren() ]
+        self.assertEquals("xsd:int",      child_types[ 0])
+        self.assertEquals("xsd:string",   child_types[ 1])
+        self.assertEquals("xsd:float",    child_types[ 2])
+        self.assertEquals("xsd:string",   child_types[ 3])
+        self.assertEquals("xsd:boolean",  child_types[ 4])
+        self.assertEquals(None,           child_types[ 5])
+        self.assertEquals(None,           child_types[ 6])
+        self.assertEquals("xsd:double",   child_types[ 7])
+        self.assertEquals("xsd:float",    child_types[ 8])
+        self.assertEquals("xsd:string",   child_types[ 9])
+        self.assertEquals("xsd:string",   child_types[10])
+        self.assertEquals("xsd:float",    child_types[11])
+        self.assertEquals("xsd:long",     child_types[12])
+
+        self.assertEquals("true", root.n.get(XML_SCHEMA_NIL_ATTR))
+
+        for c in root.getiterator():
+            self.assertEquals(None, c.get(objectify.PYTYPE_ATTRIBUTE))
 
     def test_change_pytype_attribute(self):
         XML = self.XML
@@ -596,6 +1042,16 @@ class ObjectifyTestCase(HelperTestCase):
         path = objectify.ObjectPath( "root.c1[1].c2" )
         self.assertFalse(path.hasattr(root))
 
+    def test_object_path_dot(self):
+        root = self.XML(xml_str)
+        path = objectify.ObjectPath( "." )
+        self.assertEquals(root.c1.c2.text, path(root).c1.c2.text)
+
+    def test_object_path_dot_list(self):
+        root = self.XML(xml_str)
+        path = objectify.ObjectPath( [''] )
+        self.assertEquals(root.c1.c2.text, path(root).c1.c2.text)
+
     def test_object_path_dot_root(self):
         root = self.XML(xml_str)
         path = objectify.ObjectPath( ".c1.c2" )
@@ -655,9 +1111,7 @@ class ObjectifyTestCase(HelperTestCase):
                           ['root[2]', 'c1', 'c2'])
 
         self.assertRaises(ValueError, objectify.ObjectPath,
-                          ".")
-        self.assertRaises(ValueError, objectify.ObjectPath,
-                          [''])
+                          [])
         self.assertRaises(ValueError, objectify.ObjectPath,
                           ['', '', ''])
 
@@ -873,6 +1327,17 @@ class ObjectifyTestCase(HelperTestCase):
              'root.{objectified}c1.{}c2'],
             root.c1.descendantpaths('root'))
 
+    def test_pickle(self):
+        import pickle
+
+        root = self.XML(xml_str)
+        out = StringIO()
+        pickle.dump(root, out)
+
+        new_root = pickle.loads(out.getvalue())
+        self.assertEquals(
+            etree.tostring(new_root),
+            etree.tostring(root))
 
 def test_suite():
     suite = unittest.TestSuite()
